import regress
from unicorn import *


class VldrPcInsn(regress.RegressTest):

    def runTest(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)

        uc.mem_map(0x1000, 0x1000)
        uc.mem_write(0x1000, b'\xed\x9f\x8a\x3d')  # vldr s16, [pc, #244]

        with self.assertRaises(UcError) as ex:
            uc.emu_start(0x1000, 0x1004)

        self.assertEqual(UC_ERR_INSN_INVALID, ex.exception.errno)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

CODE = (
    b'\x9b\xd9\x3c\x24'     #  fstcw  WORD PTR [esp]
    b'\x59'                 #  pop    ecx
)

BASE = 0x00000000
STACK = 0x00000f00


def hook_mem_write(uc, access, address, size, value, user_data):
    regress.logger.debug("mem WRITE to: %#x, size = %u, value = %#x", address, size, value)
    return True


class FpuWrite(regress.RegressTest):

    def runTest(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        mu.mem_map(BASE, 0x1000)
        mu.mem_write(BASE, CODE)
        mu.reg_write(UC_X86_REG_ESP, STACK)

        mu.hook_add(UC_HOOK_MEM_WRITE, hook_mem_write)
        mu.emu_start(BASE, BASE + len(CODE), count=2)

        self.assertSequenceEqual(b'\x00' * 2, mu.mem_read(STACK, 2))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for SPARC of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>
# Python sample ported by Loi Anh Tuan <loianhtuan@gmail.com>

from unicorn import *
from unicorn.sparc_const import *

# code to be emulated
SPARC_CODE = b"\x86\x00\x40\x02"  # add %g1, %g2, %g3;
# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test SPARC
def test_sparc():
    print("Emulate SPARC code")
    try:
        # Initialize emulator in SPARC EB mode
        mu = Uc(UC_ARCH_SPARC, UC_MODE_SPARC32 | UC_MODE_BIG_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, SPARC_CODE)

        # initialize machine registers
        mu.reg_write(UC_SPARC_REG_G1, 0x1230)
        mu.reg_write(UC_SPARC_REG_G2, 0x6789)
        mu.reg_write(UC_SPARC_REG_G3, 0x5555)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(SPARC_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        g3 = mu.reg_read(UC_SPARC_REG_G3)
        print(">>> G3 = 0x%x" % g3)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_sparc()



import regress
from unicorn import *
from unicorn.x86_const import *


class BadRam(regress.RegressTest):
    def runTest(self):
        PAGE_SIZE = 0x5000
        CODE_ADDR = 0x400000
        RSP_ADDR = 0x200000

        CODE = b"\xCA\x24\x5D"  # retf 0x5d24

        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        mu.mem_map(CODE_ADDR, PAGE_SIZE)
        mu.mem_map(RSP_ADDR, PAGE_SIZE)

        mu.mem_write(CODE_ADDR, CODE)
        mu.reg_write(UC_X86_REG_RSP, RSP_ADDR)

        # make sure we bump into an exception
        with self.assertRaises(UcError) as raisedEx:
            mu.emu_start(CODE_ADDR, CODE_ADDR + PAGE_SIZE)

        # make sure it is an exception with the errno we expect
        self.assertEqual(raisedEx.exception.errno, UC_ERR_READ_UNMAPPED)


if __name__ == '__main__':
    regress.main()



import sys
import unittest
import regress
from unicorn import *
from unicorn.mips_const import *


class Mips64(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        # Two instructions:
        #   daddu  $gp, $gp, $ra    # a 64-bit instruction. This is important - it ensures the selected CPU model is 64-bit, otherwise it would crash
        #   move   $t1, $v0

        code = b"\x03\x9f\xe0\x2d" + b"\x00\x40\x48\x25"

        uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS64 + UC_MODE_BIG_ENDIAN)
        # For MIPS64 to be able to reference addresses >= 0x80000000, you need to enable the virtual TLB
        # See https://github.com/unicorn-engine/unicorn/pull/2111 for more details
        uc.ctl_set_tlb_mode(UC_TLB_VIRTUAL)

        ADDRESS = 0x0120003000

        uc.reg_write(UC_MIPS_REG_PC, ADDRESS)
        uc.reg_write(UC_MIPS_REG_GP, 0x123)
        uc.reg_write(UC_MIPS_REG_RA, 0x456)

        uc.mem_map(ADDRESS, 4 * 1024)
        uc.mem_write(ADDRESS, code)

        # This will raise an exception if MIPS64 fails
        uc.emu_start(ADDRESS, 0, count=2)

        self.assertEqual(uc.reg_read(UC_MIPS_REG_PC),0x0120003000 + 8)


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for MIPS of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>
# Python sample ported by Loi Anh Tuan <loianhtuan@gmail.com>

from unicorn import *
from unicorn.mips_const import *

# code to be emulated
MIPS_CODE_EB = b"\x34\x21\x34\x56"  # ori $at, $at, 0x3456;
MIPS_CODE_EL = b"\x56\x34\x21\x34"  # ori $at, $at, 0x3456;

# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test MIPS EB
def test_mips_eb():
    print("Emulate MIPS code (big-endian)")
    try:
        # Initialize emulator in MIPS32 + EB mode
        mu = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_BIG_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, MIPS_CODE_EB)

        # initialize machine registers
        mu.reg_write(UC_MIPS_REG_1, 0x6789)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(MIPS_CODE_EB))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        r1 = mu.reg_read(UC_MIPS_REG_1)
        print(">>> R1 = 0x%x" % r1)

    except UcError as e:
        print("ERROR: %s" % e)


# Test MIPS EL
def test_mips_el():
    print("Emulate MIPS code (little-endian)")
    try:
        # Initialize emulator in MIPS32 + EL mode
        mu = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_LITTLE_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, MIPS_CODE_EL)

        # initialize machine registers
        mu.reg_write(UC_MIPS_REG_1, 0x6789)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(MIPS_CODE_EL))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        r1 = mu.reg_read(UC_MIPS_REG_1)
        print(">>> R1 = 0x%x" % r1)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_mips_eb()
    print("=" * 27)
    test_mips_el()



import regress
from unicorn import *
from unicorn.x86_const import *
from capstone import *

CODE = (
    b'\x48\x31\xc0'      # xor       rax,rax
    b'\x48\x0f\xc7\xf0'  # rdrand    rax
    b'\xf4'              # hlt
)

BASE = 0x100000
PAGE_SIZE = 0x1000

# max possible length of a x86 instruction
MAX_INSN_LEN = 15


def hook_invalid_insn(uc, ud):
    regress.logger.debug('entered invalid instruction handler')

    pc = uc.reg_read(UC_X86_REG_RIP)
    data = uc.mem_read(pc, MAX_INSN_LEN)

    md = Cs(CS_ARCH_X86, CS_MODE_64)
    insn = next(md.disasm(data, pc, 1))

    if insn.mnemonic == 'rdrand':
        # chosen by fair dice roll, guaranteed to be random
        rax = 4

        # set result to rax
        uc.reg_write(UC_X86_REG_RAX, rax)

        # resume emulation from next instruction
        uc.reg_write(UC_X86_REG_RIP, pc + insn.size)

        # signal uc we are ok
        return True

    # not handled, uc will crash
    return False


class TestHooks(regress.RegressTest):
    def test_invalid_insn_recover(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        mu.mem_map(BASE, PAGE_SIZE)
        mu.mem_write(BASE, CODE)

        mu.hook_add(UC_HOOK_INSN_INVALID, hook_invalid_insn)

        try:
            mu.emu_start(BASE, BASE + len(CODE))
        except UcError as ex:
            if ex.errno == UC_ERR_INSN_INVALID:
                self.fail('invalid instruction did not recover properly')

            # unexpected exception, re-raise
            raise

        self.assertNotEqual(0, mu.reg_read(UC_X86_REG_RAX))


if __name__ == '__main__':
    regress.main()



import regress
import sys
import unittest
from unicorn import Uc, UcError, UC_ARCH_X86, UC_MODE_64
from unicorn.unicorn_const import UC_TLB_VIRTUAL, UC_TLB_CPU, UC_ERR_FETCH_UNMAPPED

MAX_INTEL_INSN_SIZE = 15


class TestMem(regress.RegressTest):

    # 0x0008fffffffff000  : mappings till this point work ok
    # 0x0010000000000000  : mappings beyond this point will fail if tlb is not set to virtual

    def setUp(self):
        self.uc = Uc(UC_ARCH_X86, UC_MODE_64)

    def map_code_page(self, address, payload):
        regress.logger.debug('attempting to map code at %#018x', address)

        self.uc.mem_map(address, 0x1000)
        self.uc.mem_write(address, payload)

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def test_virt_high_mapping(self):
        """
        Mapping memory at high addresses should work when TLB mode
        is set to VIRTUAL.
        """

        base = 0x0010000000000000

        self.uc.ctl_set_tlb_mode(UC_TLB_VIRTUAL)

        for i in range(12):
            code = base << i

            self.map_code_page(code, b'\xf4')

            try:
                self.uc.emu_start(code, code + MAX_INTEL_INSN_SIZE, count=1)
            except UcError:
                self.fail('high mapping failed at %#018x' % code)

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def test_cpu_high_mapping(self):
        """
        Mapping memory at high addresses should work fail TLB mode
        is set to CPU (default).
        """

        base = 0x0010000000000000

        self.uc.ctl_set_tlb_mode(UC_TLB_CPU)

        for i in range(12):
            code = base << i

            self.map_code_page(code, b'\xf4')

            with self.assertRaises(UcError) as ex:
                self.uc.emu_start(code, code + MAX_INTEL_INSN_SIZE, count=1)

            self.assertEqual(UC_ERR_FETCH_UNMAPPED, ex.exception.errno)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

CODE = (
    b'\x8b\x83\xd4\x05\x00\x00'  # mov    eax, DWORD PTR [ebx+0x5d4]
    b'\x8b\x93\x80\x05\x00\x00'  # mov    edx, DWORD PTR [ebx+0x580]
)

BASE = 0x47bb000
PATT1 = b"\xaf\xaf\xaf\xaf"
PATT2 = b"\xbf\xbf\xbf\xbf"


class TestReadMem(regress.RegressTest):
    def runTest(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)

        uc.mem_map(BASE, 0x1000)

        uc.mem_write(BASE, CODE)
        uc.mem_write(BASE + 0x5d4, PATT1)
        uc.mem_write(BASE + 0x580, PATT2)

        uc.reg_write(UC_X86_REG_EBX, BASE)

        uc.emu_start(BASE, BASE + len(CODE))

        self.assertEqual(PATT1, uc.mem_read(BASE + 0x5d4, 4))
        self.assertEqual(PATT2, uc.mem_read(BASE + 0x580, 4))


if __name__ == '__main__':
    regress.main()



import regress
import sys
import unittest
from unicorn import *
from unicorn.x86_const import *


# Very basic testing to ensure the old api exists
@unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
class OldCtl(regress.RegressTest):
    def runTest(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)
        mu.ctl_tlb_mode(UC_TLB_CPU)
        mu.ctl_set_tlb_mode(UC_TLB_VIRTUAL)


if __name__ == '__main__':
    regress.main()



""" See https://github.com/unicorn-engine/unicorn/issues/65 """

import regress
import unicorn


class EmuStopSegFault(regress.RegressTest):

    def runTest(self):
        ADDR = 0x10101000
        mu = unicorn.Uc(unicorn.UC_ARCH_X86, unicorn.UC_MODE_32)
        mu.mem_map(ADDR, 1024 * 4)
        mu.mem_write(ADDR, b'\x41')
        mu.emu_start(ADDR, ADDR + 1, count=1)
        # The following should not trigger a null pointer dereference
        self.assertEqual(None, mu.emu_stop())


if __name__ == '__main__':
    regress.main()



""" https://github.com/unicorn-engine/unicorn/issues/334 """

import regress
from unicorn import *
from unicorn.x86_const import *

ADDRESS = 0x8048000
STACK_ADDRESS = 0xffff000
STACK_SIZE = 4096

CODE = (
    b'\x31\xDB'     #  xor     ebx, ebx
    b'\x53'         #  push    ebx
    b'\x43'         #  inc     ebx
    b'\x53'         #  push    ebx
    b'\x6A\x02'     #  push    2
    b'\x6A\x66'     #  push    66h
    b'\x58'         #  pop     eax
    b'\x89\xE1'     #  mov     ecx, esp
    b'\xCD\x80'     #  int     80h
)

EP = ADDRESS + 0x54


def hook_code(mu, address, size, user_data):
    regress.logger.debug(">>> Tracing instruction at %#x, instruction size = %u", address, size)


class HookCodeAddDelTest(regress.RegressTest):
    def runTest(self):
        emu = Uc(UC_ARCH_X86, UC_MODE_32)
        emu.mem_map(ADDRESS, 0x1000)
        emu.mem_write(EP, CODE)

        emu.mem_map(STACK_ADDRESS, STACK_SIZE)
        emu.reg_write(UC_X86_REG_ESP, STACK_ADDRESS + STACK_SIZE)

        # UC_HOOK_CODE hook will work even after deletion
        i = emu.hook_add(UC_HOOK_CODE, hook_code, None)
        emu.hook_del(i)

        emu.emu_start(EP, EP + len(CODE), count=3)
        regress.logger.debug("EIP: %#x", emu.reg_read(UC_X86_REG_EIP))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for S390x of Unicorn.

from unicorn import *
from unicorn.s390x_const import *

# lr %r2, %r3
S390X_CODE = b"\x18\x23"

# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test RISCV
def test_s390x():
    print("Emulate S390X code")
    try:
        # Initialize emulator in big endian mode
        mu = Uc(UC_ARCH_S390X, UC_MODE_BIG_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, S390X_CODE)

        # initialize machine registers
        mu.reg_write(UC_S390X_REG_R3, 0x7890)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(S390X_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        r2 = mu.reg_read(UC_S390X_REG_R2)
        r3 = mu.reg_read(UC_S390X_REG_R3)
        print(">>> R2 = 0x%x" % r2)
        print(">>> R3 = 0x%x" % r3)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_s390x()



# Mariano Graziano

import regress
import sys
import unittest
from unicorn import *
from unicorn.x86_const import *

# set rdx to either 0xbabe or 0xc0ca, based on a comparison.
# rdx would never be set to 0xbabe unless we set zf to 1
CODE = (
    b"\x48\x31\xc0"                                 #       xor       rax, rax
    b"\x48\xb8\x04\x00\x00\x00\x00\x00\x00\x00"     #  03:  movabs    rax, 0x4
    b"\x48\x3d\x05\x00\x00\x00"                     #  0d:  cmp       rax, 0x5      <-- never true, zf is cleared
    b"\x74\x05"                                     #  13:  je        0x1a
    b"\xe9\x0f\x00\x00\x00"                         #       jmp       0x29
    b"\x48\xba\xbe\xba\x00\x00\x00\x00\x00\x00"     #  1a:  movabs    rdx, 0xbabe   <-- never reached unless we set zf
    b"\xe9\x0f\x00\x00\x00"                         #       jmp       0x38
    b"\x48\xba\xca\xc0\x00\x00\x00\x00\x00\x00"     #  29:  movabs    rdx, 0xc0ca
    b"\xe9\x00\x00\x00\x00"                         #       jmp       0x38
    b"\xf4"                                         #  38:  hlt
)

BASE = 0x1000000


class Jumping(regress.RegressTest):
    def clear_zf(self):
        eflags = self.uc.reg_read(UC_X86_REG_EFLAGS)

        if (eflags >> 6) & 0b1 == 0b1:
            eflags &= ~(0b1 << 6)

            regress.logger.debug("[clear_zf] clearing zero flag")
            self.uc.reg_write(UC_X86_REG_EFLAGS, eflags)

        else:
            regress.logger.debug("[clear_zf] no change needed")

    def set_zf(self):
        eflags = self.uc.reg_read(UC_X86_REG_EFLAGS)

        if (eflags >> 6) & 0b1 == 0b0:
            eflags |= (0b1 << 6)

            regress.logger.debug("[set_zf] setting zero flag")
            self.uc.reg_write(UC_X86_REG_EFLAGS, eflags)

        else:
            regress.logger.debug("[set_zf] no change needed")

    def multipath(self):
        regress.logger.debug("[multipath] - handling ZF (%s) - default", self.fixed_zf)

        if self.fixed_zf:
            self.set_zf()
        else:
            self.clear_zf()

        # BUG: eflags changes do not get reflected unless re-writing eip

    # callback for tracing basic blocks
    def hook_block(self, uc, address, size, _):
        regress.logger.debug("Reached a new basic block at %#x (%d bytes in size)", address, size)

    # callback for tracing instructions
    def hook_code(self, uc, address, size, _):
        insn = uc.mem_read(address, size)
        regress.logger.debug(">>> Tracing instruction at %#x : %s", address, insn.hex())

        regs = uc.reg_read_batch((
            UC_X86_REG_RAX, UC_X86_REG_RBX, UC_X86_REG_RCX, UC_X86_REG_RDX,
            UC_X86_REG_RSI, UC_X86_REG_RDI, UC_X86_REG_RBP, UC_X86_REG_RSP,
            UC_X86_REG_R8, UC_X86_REG_R9, UC_X86_REG_R10, UC_X86_REG_R11,
            UC_X86_REG_R12, UC_X86_REG_R13, UC_X86_REG_R14, UC_X86_REG_R15,
            UC_X86_REG_EFLAGS
        ))

        zf = (regs[16] >> 6) & 0b1

        regress.logger.debug("    RAX = %08x, R8  = %08x", regs[0], regs[8])
        regress.logger.debug("    RBX = %08x, R9  = %08x", regs[1], regs[9])
        regress.logger.debug("    RCX = %08x, R10 = %08x", regs[2], regs[10])
        regress.logger.debug("    RDX = %08x, R11 = %08x", regs[3], regs[11])
        regress.logger.debug("    RSI = %08x, R12 = %08x", regs[4], regs[12])
        regress.logger.debug("    RDI = %08x, R13 = %08x", regs[5], regs[13])
        regress.logger.debug("    RBP = %08x, R14 = %08x", regs[6], regs[14])
        regress.logger.debug("    RSP = %08x, R15 = %08x", regs[7], regs[15])
        regress.logger.debug("    EFLAGS = %08x (ZF = %d)", regs[16], zf)

        regress.logger.debug("-" * 32)
        self.multipath()
        regress.logger.debug("-" * 32)

    def setUp(self):
        # decide how to fixate zf value: 0 to clear, 1 to set
        self.fixed_zf = 1

        # Initialize emulator in X86-64bit mode
        uc = Uc(UC_ARCH_X86, UC_MODE_64)

        # map one page for this emulation
        uc.mem_map(BASE, 0x1000)

        # write machine code to be emulated to memory
        uc.mem_write(BASE, CODE)

        self.uc = uc

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        # tracing all basic blocks with customized callback
        self.uc.hook_add(UC_HOOK_BLOCK, self.hook_block)

        # tracing all instructions in range [ADDRESS, ADDRESS+0x60]
        self.uc.hook_add(UC_HOOK_CODE, self.hook_code, begin=BASE, end=BASE + 0x60)

        # emulate machine code in infinite time
        self.uc.emu_start(BASE, BASE + len(CODE))

        self.assertEqual(self.uc.reg_read(UC_X86_REG_RDX), 0xbabe,
                         "rdx contains the wrong value. eflags modification failed")


if __name__ == '__main__':
    regress.main()



# By Ryan Hileman, issue #3

import regress
import sys
import unittest
from capstone import Cs, CS_ARCH_X86, CS_MODE_64
from unicorn import *

CODE = b'\xf2\x0f\x10\x05\xaa\x12\x00\x00'


def dis(md, mem, addr):
    return '\n'.join(('%-16s %s' % (insn.mnemonic, insn.op_str) for insn in md.disasm(mem, addr)))


def hook_code(uc, addr, size, md):
    mem = uc.mem_read(addr, size)

    regress.logger.debug('instruction size: %d', size)
    regress.logger.debug('instruction: %s %s', mem, dis(md, mem, addr))
    regress.logger.debug('reference:  %s %s', CODE, dis(md, CODE, addr))


class Movsd(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        addr = 0x400000
        mu = Uc(UC_ARCH_X86, UC_MODE_64)
        md = Cs(CS_ARCH_X86, CS_MODE_64)

        mu.hook_add(UC_HOOK_CODE, hook_code, md)
        mu.mem_map(addr, 8 * 1024 * 1024)
        mu.mem_write(addr, CODE)
        mu.emu_start(addr, addr + len(CODE))


if __name__ == '__main__':
    regress.main()



# Sample code for ARM of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>
# Python sample ported by Loi Anh Tuan <loianhtuan@gmail.com>


import platform
import regress
import sys
import unittest
from unicorn import *
from unicorn.arm_const import *

# code to be emulated
ARM_CODE = (
    b"\x37\x00\xa0\xe3"  # mov      r0, #0x37
    b"\x03\x10\x42\xe0"  # sub      r1, r2, r3
)

THUMB_CODE = b"\x83\xb0"  # sub      sp, #0xc

# memory address where emulation starts
ADDRESS = 0xF0000000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    regress.logger.debug(">>> Tracing basic block at %#x, block size = %#x", address, size)


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    regress.logger.debug(">>> Tracing instruction at %#x, instruction size = %u", address, size)


class TestInitInputCrash(regress.RegressTest):
    @unittest.skipIf(sys.platform == 'win32' or platform.machine().lower() not in ('x86_64', 'arm64'), 'TO BE CHECKED!')
    def test_arm(self):
        regress.logger.debug("Emulate ARM code")

        # Initialize emulator in ARM mode
        mu = Uc(UC_ARCH_ARM, UC_MODE_ARM)

        mem_size = 2 * (1024 * 1024)
        mu.mem_map(ADDRESS, mem_size)

        stack_address = ADDRESS + mem_size
        stack_size = stack_address  # >>> here huge memory size
        mu.mem_map(stack_address, stack_size)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, ARM_CODE)

        # initialize machine registers
        mu.reg_write(UC_ARM_REG_R0, 0x1234)
        mu.reg_write(UC_ARM_REG_R2, 0x6789)
        mu.reg_write(UC_ARM_REG_R3, 0x3333)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(ARM_CODE))

        # now print out some registers
        regress.logger.debug(">>> Emulation done. Below is the CPU context")

        r0 = mu.reg_read(UC_ARM_REG_R0)
        r1 = mu.reg_read(UC_ARM_REG_R1)
        regress.logger.debug(">>> R0 = %#x", r0)
        regress.logger.debug(">>> R1 = %#x", r1)

    def test_thumb(self):
        regress.logger.debug("Emulate THUMB code")

        # Initialize emulator in thumb mode
        mu = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, THUMB_CODE)

        # initialize machine registers
        mu.reg_write(UC_ARM_REG_SP, 0x1234)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS | 0b1, ADDRESS + len(THUMB_CODE))

        # now print out some registers
        regress.logger.debug(">>> Emulation done. Below is the CPU context")

        sp = mu.reg_read(UC_ARM_REG_SP)
        regress.logger.debug(">>> SP = %#x", sp)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.mips_const import *

CODE = b'\x44\x43\xF8\x00'  # cfc1    $v1, FCSR
BASE = 0x416CB0


class TestMipsCp1(regress.RegressTest):
    def runTest(self):
        uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_BIG_ENDIAN)

        uc.mem_map(BASE & ~(0x1000 - 1), 0x1000)
        uc.mem_write(BASE, CODE)

        # set a wrong value in v1
        uc.reg_write(UC_MIPS_REG_V1, 0x0badc0de)

        uc.emu_start(BASE, BASE + len(CODE))

        # default FCSR value should be 0
        self.assertEqual(0x0000, uc.reg_read(UC_MIPS_REG_V1))


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.mips_const import *

CODE = b'\x0c\x00\x00\x00'  # syscall
BASE = 0x40000


def intr_hook(uc, intno, data):
    regress.logger.debug('interrupt=%d, v0=%d, pc=%#010x', intno, uc.reg_read(UC_MIPS_REG_V0),
                         uc.reg_read(UC_MIPS_REG_PC))


class MipsSyscall(regress.RegressTest):
    def test(self):
        uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_LITTLE_ENDIAN)

        uc.mem_map(BASE, 0x1000)
        uc.mem_write(BASE, CODE)
        uc.reg_write(UC_MIPS_REG_V0, 100)
        uc.hook_add(UC_HOOK_INTR, intr_hook)

        uc.emu_start(BASE, BASE + len(CODE))

        self.assertEqual(0x40004, uc.reg_read(UC_MIPS_REG_PC))


if __name__ == '__main__':
    regress.main()



import regress
import sys
import unittest
from unicorn import *
from unicorn.x86_const import *
from capstone import Cs, CS_ARCH_X86, CS_MODE_64, CS_MODE_32

CODE = (
    b'\xc7\x04\x24\x7f\x03\x00\x00'     #  mov      DWORD PTR [rsp],0x37f
    b'\xd9\x2c\x24'                     #  fldcw    WORD PTR [rsp]
    b'\xd9\xd0'                         #  fnop
    b'\xd9\x74\x24\x08'                 #  fnstenv  [rsp+0x8]
    b'\x59'                             #  pop      rcx
)

BASE = 0x00000000
STACK = 0x00000f00


def hook_code(uc, addr, size, user_data):
    cs = user_data
    data = uc.mem_read(addr, size)
    mnem, ops = next((insn.mnemonic, insn.op_str) for insn in cs.disasm(data, addr))

    regress.logger.debug("0x%x: %-12s %-24s", addr, mnem, ops)


class FpuIP(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def test_32(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)
        cs = Cs(CS_ARCH_X86, CS_MODE_32)

        mu.mem_map(BASE, 0x1000)
        mu.mem_write(BASE, CODE)
        mu.reg_write(UC_X86_REG_ESP, STACK)
        mu.hook_add(UC_HOOK_CODE, hook_code, cs)

        mu.emu_start(BASE, BASE + len(CODE), count=5)

        self.assertSequenceEqual(b'\x7f\x03\x00\x00\x00\x00\x00\x00', mu.mem_read(STACK + 8, 8))
        self.assertSequenceEqual(b'\x55\x55\x00\x00\x00\x00\x00\x00', mu.mem_read(STACK + 16, 8))

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def test_64(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_64)
        cs = Cs(CS_ARCH_X86, CS_MODE_64)

        mu.mem_map(BASE, 0x1000)
        mu.mem_write(BASE, CODE)
        mu.reg_write(UC_X86_REG_RSP, STACK)
        mu.hook_add(UC_HOOK_CODE, hook_code, cs)

        mu.emu_start(BASE, BASE + len(CODE), count=5)

        self.assertSequenceEqual(b'\x7f\x03\x00\x00\x00\x00\x00\x00', mu.mem_read(STACK + 8, 8))
        self.assertSequenceEqual(b'\x55\x55\x00\x00\x00\x00\x00\x00', mu.mem_read(STACK + 16, 8))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for ARM of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>
# Python sample ported by Loi Anh Tuan <loianhtuan@gmail.com>

from unicorn import *
from unicorn.m68k_const import *

# code to be emulated
M68K_CODE = b"\x76\xed"  # movq #-19, %d3
# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test ARM
def test_m68k():
    print("Emulate M68K code")
    try:
        # Initialize emulator in ARM mode
        mu = Uc(UC_ARCH_M68K, UC_MODE_BIG_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, M68K_CODE)

        # initialize machine registers
        mu.reg_write(UC_M68K_REG_D3, 0x1234)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(M68K_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        a0 = mu.reg_read(UC_M68K_REG_A0)
        a1 = mu.reg_read(UC_M68K_REG_A1)
        a2 = mu.reg_read(UC_M68K_REG_A2)
        a3 = mu.reg_read(UC_M68K_REG_A3)
        a4 = mu.reg_read(UC_M68K_REG_A4)
        a5 = mu.reg_read(UC_M68K_REG_A5)
        a6 = mu.reg_read(UC_M68K_REG_A6)
        a7 = mu.reg_read(UC_M68K_REG_A7)
        d0 = mu.reg_read(UC_M68K_REG_D0)
        d1 = mu.reg_read(UC_M68K_REG_D1)
        d2 = mu.reg_read(UC_M68K_REG_D2)
        d3 = mu.reg_read(UC_M68K_REG_D3)
        d4 = mu.reg_read(UC_M68K_REG_D4)
        d5 = mu.reg_read(UC_M68K_REG_D5)
        d6 = mu.reg_read(UC_M68K_REG_D6)
        d7 = mu.reg_read(UC_M68K_REG_D7)
        pc = mu.reg_read(UC_M68K_REG_PC)
        sr = mu.reg_read(UC_M68K_REG_SR)
        print(">>> A0 = 0x%x\t\t>>> D0 = 0x%x" % (a0, d0))
        print(">>> A1 = 0x%x\t\t>>> D1 = 0x%x" % (a1, d1))
        print(">>> A2 = 0x%x\t\t>>> D2 = 0x%x" % (a2, d2))
        print(">>> A3 = 0x%x\t\t>>> D3 = 0x%x" % (a3, d3))
        print(">>> A4 = 0x%x\t\t>>> D4 = 0x%x" % (a4, d4))
        print(">>> A5 = 0x%x\t\t>>> D5 = 0x%x" % (a5, d5))
        print(">>> A6 = 0x%x\t\t>>> D6 = 0x%x" % (a6, d6))
        print(">>> A7 = 0x%x\t\t>>> D7 = 0x%x" % (a7, d7))
        print(">>> PC = 0x%x" % pc)
        print(">>> SR = 0x%x" % sr)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_m68k()



import regress
import sys
import unittest
from unicorn import *
from unicorn.sparc_const import *

PAGE_SIZE = 1 * 1024 * 1024

CODE = (
    b"\x80\x00\x20\x01"  # add %g0, 1, %g0
    b"\x82\x00\x60\x01"  # add %g1, 1, %g1
    b"\x84\x00\xA0\x01"  # add %g2, 1, %g2
    b"\x86\x00\xE0\x01"  # add %g3, 1, %g3
    b"\x88\x01\x20\x01"  # add %g4, 1, %g4
    b"\x8A\x01\x60\x01"  # add %g5, 1, %g5
    b"\x8C\x01\xA0\x01"  # add %g6, 1, %g6
    b"\x8E\x01\xE0\x01"  # add %g7, 1, %g7
    b"\x90\x02\x20\x01"  # add %o0, 1, %o0
    b"\x92\x02\x60\x01"  # add %o1, 1, %o1
    b"\x94\x02\xA0\x01"  # add %o2, 1, %o2
    b"\x96\x02\xE0\x01"  # add %o3, 1, %o3
    b"\x98\x03\x20\x01"  # add %o4, 1, %o4
    b"\x9A\x03\x60\x01"  # add %o5, 1, %o5
    b"\x9C\x03\xA0\x01"  # add %sp, 1, %sp
    b"\x9E\x03\xE0\x01"  # add %o7, 1, %o7
    b"\xA0\x04\x20\x01"  # add %l0, 1, %l0
    b"\xA2\x04\x60\x01"  # add %l1, 1, %l1
    b"\xA4\x04\xA0\x01"  # add %l2, 1, %l2
    b"\xA6\x04\xE0\x01"  # add %l3, 1, %l3
    b"\xA8\x05\x20\x01"  # add %l4, 1, %l4
    b"\xAA\x05\x60\x01"  # add %l5, 1, %l5
    b"\xAC\x05\xA0\x01"  # add %l6, 1, %l6
    b"\xAE\x05\xE0\x01"  # add %l7, 1, %l7
    b"\xB0\x06\x20\x01"  # add %i0, 1, %i0
    b"\xB2\x06\x60\x01"  # add %i1, 1, %i1
    b"\xB4\x06\xA0\x01"  # add %i2, 1, %i2
    b"\xB6\x06\xE0\x01"  # add %i3, 1, %i3
    b"\xB8\x07\x20\x01"  # add %i4, 1, %i4
    b"\xBA\x07\x60\x01"  # add %i5, 1, %i5
    b"\xBC\x07\xA0\x01"  # add %fp, 1, %fp
    b"\xBE\x07\xE0\x01"  # add %i7, 1, %i7
)

BASE = 0x00000000


def hook_code(uc, addr, size, ud):
    regress.logger.debug("executing at 0x%04x", uc.reg_read(UC_SPARC_REG_PC))


class TestSparc32RegRead(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        uc = Uc(UC_ARCH_SPARC, UC_MODE_SPARC32 | UC_MODE_BIG_ENDIAN)

        uc.reg_write(UC_SPARC_REG_SP, 100)
        uc.reg_write(UC_SPARC_REG_FP, 200)

        uc.mem_map(BASE, PAGE_SIZE)
        uc.mem_write(BASE, CODE)

        uc.hook_add(UC_HOOK_CODE, hook_code)
        uc.emu_start(BASE, len(CODE))

        self.assertEqual(0, uc.reg_read(UC_SPARC_REG_G0))  # G0 is always zero
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_G1))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_G2))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_G3))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_G4))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_G5))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_G6))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_G7))

        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_O0))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_O1))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_O2))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_O3))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_O4))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_O5))
        self.assertEqual(101, uc.reg_read(UC_SPARC_REG_O6))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_O7))

        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L0))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L1))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L2))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L3))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L4))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L5))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L6))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_L7))

        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_I0))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_I1))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_I2))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_I3))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_I4))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_I5))
        self.assertEqual(201, uc.reg_read(UC_SPARC_REG_I6))
        self.assertEqual(1, uc.reg_read(UC_SPARC_REG_I7))

        # BUG: PC seems to get reset to 4 when done executing
        self.assertEqual(4 * 32, uc.reg_read(UC_SPARC_REG_PC))  # make sure we executed all instructions
        self.assertEqual(101, uc.reg_read(UC_SPARC_REG_SP))
        self.assertEqual(201, uc.reg_read(UC_SPARC_REG_FP))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for RISCV of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>

from unicorn import *
from unicorn.riscv_const import *

'''
$ cstool riscv64 1305100093850502
 0  13 05 10 00  addi	a0, zero, 1
 4  93 85 05 02  addi	a1, a1, 0x20
'''
RISCV_CODE = b"\x13\x05\x10\x00\x93\x85\x05\x02"

# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test RISCV
def test_riscv():
    print("Emulate RISCV code")
    try:
        # Initialize emulator in RISCV32 mode
        mu = Uc(UC_ARCH_RISCV, UC_MODE_RISCV32)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, RISCV_CODE)

        # initialize machine registers
        mu.reg_write(UC_RISCV_REG_A0, 0x1234)
        mu.reg_write(UC_RISCV_REG_A1, 0x7890)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(RISCV_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        a0 = mu.reg_read(UC_RISCV_REG_A0)
        a1 = mu.reg_read(UC_RISCV_REG_A1)
        print(">>> A0 = 0x%x" % a0)
        print(">>> A1 = 0x%x" % a1)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_riscv()



""" https://github.com/unicorn-engine/unicorn/issues/165 """

import regress
from unicorn import *


class TestHook(regress.RegressTest):
    def test_excessive_hooks(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        for _ in range(1337):
            mu.hook_add(UC_HOOK_MEM_READ_UNMAPPED, lambda *args, **kwargs: None)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

PAGE_SIZE = 0x1000

CODE = b'\xf3\xaa'  # rep stosb
BASE = 0x00000000


class TestRep(regress.RegressTest):

    def runTest(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        mu.mem_map(BASE, PAGE_SIZE)
        mu.mem_write(BASE, CODE)

        mu.reg_write(UC_X86_REG_ECX, 8)
        mu.reg_write(UC_X86_REG_ESI, 0x10)
        mu.reg_write(UC_X86_REG_EDI, 0x20)

        def __hook_code(uc, addr, size, ud):
            regress.logger.debug('iterations remaining: %d', uc.reg_read(UC_X86_REG_ECX))

        mu.hook_add(UC_HOOK_CODE, __hook_code)

        mu.emu_start(BASE, len(CODE))

        self.assertEqual(0, mu.reg_read(UC_X86_REG_ECX))


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

CODE = (
    b'\x48\xc7\xc0\x03\x00\x00\x00'     # 0x1000:    mov      rax, 3
    b'\x0f\x05'                         # 0x1007:    syscall
    b'\x48\xc7\xc7\x00\x40\x00\x00'     # 0x1009:    mov      rdi, 0x4000
    b'\x48\x89\x07'                     # 0x1010:    mov      [rdi], rdx
    b'\x48\x8b\x07'                     # 0x1013:    mov      rdx, [rdi]
    b'\x48\x83\xc2\x01'                 # 0x1016:    add      rdx, 1
)

BASE = 0x00001000
SCRATCH = 0x00004000


class SingleStepper:
    def __init__(self, uc, test):
        self.uc = uc
        self.hits = 0
        self.test = test

    def _stop_hook(self, uc, address, *args, **kwargs):
        self.hits += 1

        if self.hits > 1:
            self.test.assertEqual(2, self.hits, "HOOK_CODE invoked too many times")
            uc.emu_stop()

    def step(self):
        self.hits = 0
        h = self.uc.hook_add(UC_HOOK_CODE, self._stop_hook)

        try:
            pc = self.uc.reg_read(UC_X86_REG_RIP)
            self.uc.emu_start(pc, pc + 0x20)
        finally:
            self.uc.hook_del(h)


def showpc(mu):
    regress.logger.debug("pc: %#x", mu.reg_read(UC_X86_REG_RIP))


class HookCodeStopEmuTest(regress.RegressTest):
    def test_hook_code_stop_emu(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        # base of CODE
        mu.mem_map(BASE, 0x1000)
        mu.mem_write(BASE, CODE)

        # scratch, used by CODE
        mu.mem_map(SCRATCH, 0x1000)

        mu.reg_write(UC_X86_REG_RDX, 0x1)
        mu.reg_write(UC_X86_REG_RIP, BASE)

        stepper = SingleStepper(mu, self)
        showpc(mu)
        self.assertEqual(BASE + 0x0, mu.reg_read(UC_X86_REG_RIP), "Unexpected starting PC")

        stepper.step()
        showpc(mu)
        self.assertEqual(BASE + 0x7, mu.reg_read(UC_X86_REG_RIP), "Emulator failed to stop after one instruction")

        stepper.step()
        showpc(mu)
        self.assertEqual(BASE + 0x9, mu.reg_read(UC_X86_REG_RIP), "Emulator failed to stop after one instruction")


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.arm_const import *

MAIN_ADDRESS = 0x8d68
ADDRESS = MAIN_ADDRESS & ~(0x1000 - 1)
STACK_ADDR = ADDRESS + 0x1000


class BxTwiceTest(regress.RegressTest):
    def runTest(self):
        # code to be emulated
        code = {
            0x8cd4: (
                b'\x04\xb0\x2d\xe5'  # 8cd4     push     {r11}
                b'\x00\xb0\x8d\xe2'  # 8cd8     add      r11, sp, #0
                b'\x0f\x30\xa0\xe1'  # 8cdc     mov      r3, pc
                b'\x03\x00\xa0\xe1'  # 8ce0     mov      r0, r3
                b'\x00\xd0\x4b\xe2'  # 8ce4     sub      sp, r11, #0
                b'\x04\xb0\x9d\xe4'  # 8ce8     pop      {r11}
                b'\x1e\xff\x2f\xe1'  # 8cec     bx       lr
            ),
            0x8cf0: (
                b'\x04\xb0\x2d\xe5'  # 8cf0     push     {r11}
                b'\x00\xb0\x8d\xe2'  # 8cf4     add      r11, sp, #0
                b'\x04\x60\x2d\xe5'  # 8cf8     push     {r6}
                b'\x01\x60\x8f\xe2'  # 8cfc     add      r6, pc, $1
                b'\x16\xff\x2f\xe1'  # 8d00     bx       r6
                                     #           .thumb
                b'\x7b\x46'          # 8d04     mov      r3, pc
                b'\x03\xf1\x08\x03'  # 8d06     add      r3, $0x8        # elicn: used to be $0x4 but it kept failing
                b'\x08\xb4'          # 8d0a     push     {r3}
                b'\x00\xbd'          # 8d0c     pop      {pc}
                b'\x00\x00'          # 8d0e     (alignment)
                                     #           .arm
                b'\x04\x60\x9d\xe4'  # 8d10     pop      {r6}
                b'\x03\x00\xa0\xe1'  # 8d14     mov      r0, r3
                b'\x00\xd0\x4b\xe2'  # 8d18     sub      sp, r11, #0
                b'\x04\xb0\x9d\xe4'  # 8d1c     pop      {r11}
                b'\x1e\xff\x2f\xe1'  # 8d20     bx       lr
            ),
            0x8d24: (  # elicn: used to be 0x8d20 but it caused this block to overlap with the previous one
                b'\x04\xb0\x2d\xe5'  # 8d24     push     {r11}
                b'\x00\xb0\x8d\xe2'  # 8d28     add      r11, sp, #0
                b'\x0e\x30\xa0\xe1'  # 8d2c     mov      r3, lr
                b'\x03\x00\xa0\xe1'  # 8d20     mov      r0, r3
                b'\x00\xd0\x4b\xe2'  # 8d34     sub      sp, r11, #0
                b'\x04\xb0\x9d\xe4'  # 8d38     pop      {r11}
                b'\x1e\xff\x2f\xe1'  # 8d3c     bx       lr
            ),
            0x8d68: (
                b'\xd9\xff\xff\xeb'  # 8d68     bl       0x8cd4      <-- MAIN_ADDRESS
                b'\x00\x40\xa0\xe1'  # 8d6c     mov      r4, r0
                b'\xde\xff\xff\xeb'  # 8d70     bl       0x8cf0
                b'\x00\x30\xa0\xe1'  # 8d74     mov      r3, r0
                b'\x03\x40\x84\xe0'  # 8d78     add      r4, r4, r3
                b'\xe8\xff\xff\xeb'  # 8d7c     bl       0x8d24
                b'\x00\x30\xa0\xe1'  # 8d80     mov      r3, r0
                b'\x03\x20\x84\xe0'  # 8d84     add      r2, r4, r3
            )
        }

        mu = Uc(UC_ARCH_ARM, UC_MODE_ARM)

        mu.mem_map(ADDRESS, 0x1000)

        # write machine code to be emulated to memory
        for addr, c in code.items():
            regress.logger.debug("Writing %d bytes to %#x", len(c), addr)
            mu.mem_write(addr, c)

        # initialize machine registers
        mu.reg_write(UC_ARM_REG_PC, MAIN_ADDRESS)
        mu.reg_write(UC_ARM_REG_SP, STACK_ADDR)

        regress.logger.debug("Starting emulation")

        # trace code only if we are debugging it
        if regress.logger.isEnabledFor(regress.logging.DEBUG):
            def __hook_code(uc, addr, size, _):
                cpsr, r0, r3, r4, r6 = uc.reg_read_batch((
                    UC_ARM_REG_CPSR,
                    UC_ARM_REG_R0,
                    UC_ARM_REG_R3,
                    UC_ARM_REG_R4,
                    UC_ARM_REG_R6
                ))

                is_thumb = (cpsr >> 5) & 0b1

                opcode = uc.mem_read(addr, size).hex()

                regress.logger.debug(
                    "%-2s PC = %#06x | opcode = %-8s    [R0 = %#06x, R3 = %#06x, R4 = %#07x, R6 = %#06x]",
                    "T" if is_thumb else "", addr, opcode, r0, r3, r4, r6
                )

            mu.hook_add(UC_HOOK_CODE, __hook_code)

        mu.emu_start(MAIN_ADDRESS, MAIN_ADDRESS + len(code[MAIN_ADDRESS]))

        regress.logger.debug("Emulation done")

        self.assertEqual(0x8ce4 + 0x8d10 + 0x8d80, mu.reg_read(UC_ARM_REG_R2))


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

NOPSLED = b"\x90" * 5


class TestSetIP(regress.RegressTest):
    def runTest(self):
        # execution history
        history = []

        def __code_hook(uc, addr, size, ud):
            # track execution history
            history.append(addr)

            if len(history) == 5:
                uc.emu_stop()
            else:
                uc.reg_write(UC_X86_REG_RIP, 0x1800)

        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        mu.mem_map(0x1000, 0x1000)
        mu.mem_write(0x1000, NOPSLED)
        mu.mem_write(0x1800, NOPSLED)

        mu.hook_add(UC_HOOK_CODE, __code_hook)
        mu.emu_start(0x1000, 0x1800 + 1)

        self.assertListEqual([0x1000] + [0x1800] * 4, history)


if __name__ == '__main__':
    regress.main()



"""
See https://github.com/unicorn-engine/unicorn/issues/161

Ensure that constants which are specified via a typedef, rather than an enum,
are included in the bindings by the script for autogenerating mappings for
constants.
"""

import regress
import unicorn


class TestTypedefs(regress.RegressTest):
    def runTest(self):
        prop = 'UC_HOOK_MEM_UNMAPPED'

        try:
            getattr(unicorn, prop)
        except AttributeError:
            self.fail("Definition for %s not generated" % prop)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

CODE = (
    b'\x8B\x74\x01\x28'  # mov esi, dword ptr [ecx + eax + 0x28]  mapped: 0x1000
    b'\x03\xF0'          # add esi, eax                                   0x1004
    b'\x8D\x45\xFC'      # lea eax, dword ptr [ebp - 4]                   0x1006
    b'\x50'              # push eax                                       0x1009
    b'\x6A\x40'          # push 0x40                                      0x100A
    b'\x6A\x10'          # push 0x10                                      0x100C
    b'\x56'              # push esi                                       0x100E
)
BASE = 0x1000
STACK = 0x4000


class EmuClearErrorsTest(regress.RegressTest):

    def test_hook_code_stop_emu(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        # base of CODE
        mu.mem_map(BASE, 0x1000)
        mu.mem_write(BASE, CODE)

        # base of STACK
        mu.mem_map(STACK, 0x1000)
        mu.mem_write(STACK, b"\x00" * 0x1000)

        mu.reg_write(UC_X86_REG_EIP, BASE)
        mu.reg_write(UC_X86_REG_ESP, STACK + 0x1000 - 8)
        mu.reg_write(UC_X86_REG_EBP, STACK + 0x1000 - 8)
        mu.reg_write(UC_X86_REG_ECX, 0x0)
        mu.reg_write(UC_X86_REG_EAX, 0x0)

        # we only expect the following instruction to execute,
        #  and it will fail, because it accesses unmapped memory.
        # mov esi, dword ptr [ecx + eax + 0x28]    mapped: 0x1000

        with self.assertRaises(UcError) as ex:
            mu.emu_start(BASE, BASE + len(CODE), count=1)

        self.assertEqual(UC_ERR_READ_UNMAPPED, ex.exception.errno)

        regress.logger.debug("pc: %#x", mu.reg_read(UC_X86_REG_EIP))

        # now, we want to reuse the emulator, and keep executing
        #  from the next instruction

        # we expect the following instructions to execute
        #   add esi, eax                                   0x1004
        #   lea eax, dword ptr [ebp - 4]                   0x1006
        #   push eax                                       0x1009
        #   push 0x40                                      0x100A
        #   push 0x10                                      0x100C
        #   push esi                                       0x100E
        mu.emu_start(BASE + 0x4, BASE + len(CODE))

        regress.logger.debug("pc: %#x", mu.reg_read(UC_X86_REG_EIP))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for Unicorn.
# By Lazymio(@wtdcode), 2021

from unicorn import *
from unicorn.x86_const import *
from datetime import datetime


def test_uc_ctl_read():
    uc = Uc(UC_ARCH_X86, UC_MODE_32)

    print("Reading some properties by uc_ctl.")

    arch = uc.ctl_get_arch()

    mode = uc.ctl_get_mode()

    page_size = uc.ctl_get_page_size()

    timeout = uc.ctl_get_timeout()

    print(">>> arch={arch} mode={mode} page size={page_size} timeout={timeout}".format(arch=arch, mode=mode,
                                                                                       page_size=page_size,
                                                                                       timeout=timeout))


def time_emulation(uc, start, end):
    n = datetime.now()

    uc.emu_start(start, end)

    return (datetime.now() - n).total_seconds() * 1e6


# TODO: Check if worth adapting the ctl_request_cache method for py2 bindings
def test_uc_ctl_tb_cache():
    # Initialize emulator in X86-32bit mode
    uc = Uc(UC_ARCH_X86, UC_MODE_32)
    addr = 0x10000

    # Fill the code buffer with NOP.
    code = b"\x90" * 8 * 512

    print("Controlling the TB cache in a finer granularity by uc_ctl.")

    uc.mem_map(addr, 0x10000)

    # Write our code to the memory.
    uc.mem_write(addr, code)

    # Do emulation without any cache.
    standard = time_emulation(uc, addr, addr + len(code))

    # Now we request cache for all TBs.
    for i in range(8):
        tb = uc.ctl_request_cache(addr + i * 512)
        print(">>> TB is cached at {:#x} which has {} instructions with {} bytes".format(tb[0], tb[1], tb[2]))

    # Do emulation with all TB cached.
    cached = time_emulation(uc, addr, addr + len(code))

    # Now we clear cache for all TBs.
    for i in range(8):
        uc.ctl_remove_cache(addr + i * 512, addr + i * 512 + 1)

    evicted = time_emulation(uc, addr, addr + len(code))

    print(">>> Run time: First time {standard}, Cached: {cached}, Cached evicted: {evicted}".format(standard=standard,
                                                                                                    cached=cached,
                                                                                                    evicted=evicted))


def trace_new_edge(uc, cur, prev, data):
    print(">>> Getting a new edge from {:#x} to {:#x}".format(prev.pc + prev.size - 1, cur.pc))


def trace_tcg_sub(uc, address, arg1, arg2, size, data):
    print(">>> Get a tcg sub opcode at {address:#x} with args: {arg1} and {arg2}".format(address=address, arg1=arg1,
                                                                                         arg2=arg2))


# TODO: Check if worth adapting the hook_add method for py2 bindings
def test_uc_ctl_exits():
    uc = Uc(UC_ARCH_X86, UC_MODE_32)
    addr = 0x1000
    #   cmp eax, 0;
    #   jg lb;
    #   inc eax;
    #   nop;
    # lb:
    #   inc ebx;
    #   nop;
    code = b"\x83\xf8\x00\x7f\x02\x40\x90\x43\x90"
    exits = [addr + 6, addr + 8]

    print("Using multiple exits by uc_ctl")

    uc.mem_map(addr, 0x1000)

    # Write our code to the memory.
    uc.mem_write(addr, code)

    # We trace if any new edge is generated.
    uc.hook_add(UC_HOOK_EDGE_GENERATED, trace_new_edge)

    # Trace cmp instruction.
    uc.hook_add(UC_HOOK_TCG_OPCODE, trace_tcg_sub, aux1=UC_TCG_OP_SUB, aux2=UC_TCG_OP_FLAG_CMP)

    uc.ctl_exits_enabled(True)

    uc.ctl_set_exits(exits)

    # This should stop at ADDRESS + 6 and increase eax, even though we don't provide an exit.
    uc.emu_start(addr, 0)

    eax = uc.reg_read(UC_X86_REG_EAX)
    ebx = uc.reg_read(UC_X86_REG_EBX)

    print(">>> eax = {eax:#x} and ebx = {ebx:#x} after the first emulation".format(eax=eax, ebx=ebx))

    # This should stop at ADDRESS + 8, even though we don't provide an exit.
    uc.emu_start(addr, 0)

    eax = uc.reg_read(UC_X86_REG_EAX)
    ebx = uc.reg_read(UC_X86_REG_EBX)

    print(">>> eax = {eax:#x} and ebx = {ebx:#x} after the first emulation".format(eax=eax, ebx=ebx))


if __name__ == "__main__":
    test_uc_ctl_read()
    print("=" * 32)
    test_uc_ctl_tb_cache()
    print("=" * 32)
    test_uc_ctl_exits()



import binascii
import regress
from unicorn import *
from unicorn.x86_const import *


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    # invalid instruction?
    if size == 0xf1f1f1f1:
        return

    regress.logger.debug("[%#x] = %s", address, binascii.hexlify(uc.mem_read(address, size)))


# callback for tracing Linux interrupt
def hook_intr(uc, intno, user_data):
    # only handle Linux syscall
    rip = uc.reg_read(UC_X86_REG_RIP)

    regress.logger.debug("[%#x]: got interrupt %#x", rip, intno)
    regress.logger.debug("  EAX = %#010x", uc.reg_read(UC_X86_REG_EAX))
    regress.logger.debug("  EBX = %#010x", uc.reg_read(UC_X86_REG_EBX))
    regress.logger.debug("  ECX = %#010x", uc.reg_read(UC_X86_REG_ECX))
    regress.logger.debug("  EDX = %#010x", uc.reg_read(UC_X86_REG_EDX))

    uc.emu_stop()


class Hang(regress.RegressTest):

    def runTest(self):
        # self modifying shellcode execve('/bin/sh')
        shellcode = (
            b'\xeb\x1c'                                  #  00:   jmp    0x1e
            b'\x5a'                                      #  02:   pop    rdx
            b'\x89\xd6'                                  #  03:   mov    esi, edx
            b'\x8b\x02'                                  #  05:   mov    eax, [rdx]
            b'\x66\x3d\xca\x7d'                          #  07:   cmp    ax, 0x7dca
            b'\x75\x06'                                  #  0b:   jne    0x13
            b'\x66\x05\x03\x03'                          #  0d:   add    ax,0x303
            b'\x89\x02'                                  #  11:   mov    [rdx], eax
            b'\xfe\xc2'                                  #  13:   inc    dl
            b'\x3d\x41\x41\x41\x41'                      #  15:   cmp    eax, 0x41414141
            b'\x75\xe9'                                  #  1a:   jne    0x5
            b'\xff\xe6'                                  #  1c:   jmp    rsi
            b'\xe8\xdf\xff\xff\xff'                      #  1e:   call   0x2
            b'\x31\xd2'                                  #  23:   xor    edx, edx
            b'\x6a\x0b'                                  #  25:   push   0xb
            b'\x58'                                      #  27:   pop    rax
            b'\x99'                                      #  28:   cdq
            b'\x52'                                      #  29:   push   rdx
            b'\x68\x2f\x2f\x73\x68'                      #  2a:   push   0x68732f2f
            b'\x68\x2f\x62\x69\x6e'                      #  2f:   push   0x6e69622f
            b'\x89\xe3'                                  #  34:   mov    ebx, esp
            b'\x52'                                      #  36:   push   rdx
            b'\x53'                                      #  37:   push   rbx
            b'\x89\xe1'                                  #  38:   mov    ecx, esp
            b'\xca\x7d\x41\x41\x41\x41\x41\x41\x41\x41'  #  3a:   .db ca 7d 41 41 41 41 41 41 41 41
        )

        address = 0x00000000

        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        mu.mem_map(address, 0x1000)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # handle interrupt ourself
        mu.hook_add(UC_HOOK_INTR, hook_intr)

        # setup stack
        mu.reg_write(UC_X86_REG_RSP, 0x1000 - 8)

        # write machine code to be emulated to memory
        mu.mem_write(address, shellcode)

        regress.logger.debug('Starting emulation')

        mu.emu_start(address, address + len(shellcode))


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *

CODE = (
    b'\xb8\x00\x00\x00\x02'  #  mov eax, 0x2000000
    b'\xdb\x28'              #  fldt [eax]
    b'\xd9\xfa'              #  fsqrt
)

BASE = 0x1000000

DATA_ADDR = 0x2000000
DATA = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01'


class FldtFsqrt(regress.RegressTest):
    def test_fldt_fsqrt(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)

        uc.mem_map(BASE, 0x1000)
        uc.mem_write(BASE, CODE)

        uc.mem_map(DATA_ADDR, 0x1000)
        uc.mem_write(DATA_ADDR, DATA)

        uc.emu_start(BASE, BASE + len(CODE), 10000, 10)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import Uc, UC_ARCH_X86, UC_MODE_64, UC_HOOK_CODE

CODE = b"\x90" * 3
CODE_ADDR = 0x1000


class HookCounter:
    """Counts number of hook calls."""

    def __init__(self):
        self.hook_calls = 0

    def bad_code_hook(self, uc, address, size, data):
        self.hook_calls += 1
        raise ValueError("Something went wrong")

    def good_code_hook(self, uc, address, size, data):
        self.hook_calls += 1


class TestExceptionInHook(regress.RegressTest):

    def test_exception_in_hook(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_64)
        uc.mem_map(CODE_ADDR, 0x1000)
        uc.mem_write(CODE_ADDR, CODE)

        counter = HookCounter()
        uc.hook_add(UC_HOOK_CODE, counter.good_code_hook, begin=CODE_ADDR, end=CODE_ADDR + len(CODE))
        uc.hook_add(UC_HOOK_CODE, counter.bad_code_hook, begin=CODE_ADDR, end=CODE_ADDR + len(CODE))

        self.assertRaises(ValueError, uc.emu_start, CODE_ADDR, CODE_ADDR + len(CODE))
        # Make sure hooks calls finish before raising (hook_calls == 2)
        self.assertEqual(counter.hook_calls, 2)


if __name__ == "__main__":
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *
from struct import pack

F_GRANULARITY   = 0x8
F_PROT_32       = 0x4
F_LONG          = 0x2
F_AVAILABLE     = 0x1

A_PRESENT = 0x80

A_PRIV_3 = 0x60
A_PRIV_2 = 0x40
A_PRIV_1 = 0x20
A_PRIV_0 = 0x00

A_CODE = 0x10
A_DATA = 0x10
A_TSS  = 0x0
A_GATE = 0x0

A_DATA_WRITABLE = 0x2
A_CODE_READABLE = 0x2

A_DIR_CON_BIT = 0x4

S_GDT = 0x0
S_LDT = 0x4
S_PRIV_3 = 0x3
S_PRIV_2 = 0x2
S_PRIV_1 = 0x1
S_PRIV_0 = 0x0

CODE = b'\x65\x33\x0d\x18\x00\x00\x00'  # xor ecx, dword ptr gs:[0x18]


def create_selector(idx, flags):
    to_ret = flags
    to_ret |= idx << 3

    return to_ret


def create_gdt_entry(base, limit, access, flags):
    return pack('<Q', (
            limit & 0xffff
            | (base & 0xffffff) << 16
            | (access & 0xff) << 40
            | ((limit >> 16) & 0xf) << 48
            | (flags & 0xff) << 52
            | ((base >> 24) & 0xff) << 56
    ))


def hook_mem_read(uc, type, addr, *args):
    regress.logger.debug("%#x", addr)

    return False


CODE_ADDR = 0x40000
CODE_SIZE = 0x1000

GDT_ADDR = 0x3000
GDT_LIMIT = 0x1000
GDT_ENTRY_SIZE = 0x8

SEGMENT_ADDR = 0x5000
SEGMENT_SIZE = 0x1000


class GdtRead(regress.RegressTest):

    def test_gdt(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)
        uc.hook_add(UC_HOOK_MEM_READ_UNMAPPED, hook_mem_read)

        uc.mem_map(GDT_ADDR, GDT_LIMIT)
        uc.mem_map(SEGMENT_ADDR, SEGMENT_SIZE)
        uc.mem_map(CODE_ADDR, CODE_SIZE)

        uc.mem_write(CODE_ADDR, CODE)
        uc.mem_write(SEGMENT_ADDR + 0x18, b'AAAA')

        gdt_entry = create_gdt_entry(SEGMENT_ADDR, SEGMENT_SIZE,
                                     A_PRESENT | A_DATA | A_DATA_WRITABLE | A_PRIV_3 | A_DIR_CON_BIT, F_PROT_32)
        uc.mem_write(GDT_ADDR + 8, gdt_entry)

        uc.reg_write(UC_X86_REG_GDTR, (0, GDT_ADDR, GDT_LIMIT, 0x0))

        selector = create_selector(1, S_GDT | S_PRIV_3)
        uc.reg_write(UC_X86_REG_GS, selector)

        uc.emu_start(CODE_ADDR, CODE_ADDR + len(CODE))

        self.assertEqual(uc.reg_read(UC_X86_REG_ECX), 0x41414141)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *


class MmapSeg1(regress.RegressTest):
    def runTest(self):
        u = Uc(UC_ARCH_X86, UC_MODE_32)
        u.mem_map(0x2000, 0x1000)
        u.mem_read(0x2000, 1)

        for i in range(50):
            u = Uc(UC_ARCH_X86, UC_MODE_32)
            u.mem_map(i * 0x1000, 0x1000)
            u.mem_read(i * 0x1000, 1)

        for i in range(20):
            with self.assertRaises(UcError):
                u = Uc(UC_ARCH_X86, UC_MODE_32)
                u.mem_map(i * 0x1000, 5)
                u.mem_read(i * 0x1000, 1)


class MmapSeg2(regress.RegressTest):
    def runTest(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)
        uc.mem_map(0x0000, 0x2000)
        uc.mem_map(0x2000, 0x4000)
        uc.mem_write(0x1000, b' ' * 0x1004)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *

CODE = (
    b'\x00\x00\x8a\xe0'     #       ADD R0, R10, R0
    b'\xff\xff\xff\xea'     #       B L0
    b'\x00\x00\x8a\xe0'     #  L0:  ADD R0, R10, R0     <-- we stop here, the first instruction of the next TB
)

BASE = 0x1000


class TestARMFirstInsn(regress.RegressTest):
    def runTest(self):
        mu = Uc(UC_ARCH_ARM, UC_MODE_ARM)

        mu.mem_map(BASE, 0x1000)
        mu.mem_write(BASE, CODE)

        mu.emu_start(BASE, BASE + len(CODE) - 4)


if __name__ == '__main__':
    regress.main()



import gc
import regress
import weakref
from unicorn import *
from unicorn.x86_const import *

ADDRESS = 0x8048000
STACK_ADDRESS = 0xffff000
STACK_SIZE = 0x1000

CODE = (
    b"\x31\xDB"     #  xor      ebx, ebx
    b"\x53"         #  push     ebx
    b"\x43"         #  inc      ebx
    b"\x53"         #  push     ebx
    b"\x6A\x02"     #  push     2
    b"\x6A\x66"     #  push     66h
    b"\x58"         #  pop      eax
    b"\x89\xE1"     #  mov      ecx, esp
    b"\xCD\x80"     #  int      80h
)

EP = ADDRESS + 0x54

# Dictionary to keep weak references to instances
instances = weakref.WeakValueDictionary()


def create_instance(key, *args, **kwargs):
    obj = Uc(*args, **kwargs)
    instances[key] = obj

    return obj


def hook_code(mu, address, size, user_data):
    regress.logger.debug(">>> Tracing instruction at %#x, instruction size = %u", address, size)


def emu_loop(key):
    emu = create_instance(key, UC_ARCH_X86, UC_MODE_32)
    emu.mem_map(ADDRESS, 0x1000)
    emu.mem_write(EP, CODE)

    emu.mem_map(STACK_ADDRESS, STACK_SIZE)
    emu.reg_write(UC_X86_REG_ESP, STACK_ADDRESS + STACK_SIZE)

    i = emu.hook_add(UC_HOOK_CODE, hook_code, None)
    emu.hook_del(i)

    emu.emu_start(EP, EP + len(CODE), count=3)
    regress.logger.debug("EIP: %#x", emu.reg_read(UC_X86_REG_EIP))



class EmuLoopReferenceTest(regress.RegressTest):
    def debug_mem(self):
        gc.collect()  # don't care about stuff that would be garbage collected properly

        self.assertEqual(len(instances), 0)

    def runTest(self):
        for i in range(5):
            emu_loop('obj%d' % i)

        self.debug_mem()


if __name__ == '__main__':
    regress.main()
